@@ -1,6 +1,6 @@
 /* IELR's inadequacy annotation list.
 
-   Copyright (C) 2009-2013 Free Software Foundation, Inc.
+   Copyright (C) 2009-2015, 2018 Free Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -546,7 +546,7 @@ AnnotationList__compute_from_inadequacies (
                     AnnotationList__insertInto (annotation_node,
                                                 &annotation_lists[s->number],
                                                 s->nitems);
-                  aver (b);
+                  aver (b); (void) b;
                 }
                 /* This aver makes sure the
                    AnnotationList__computeDominantContribution check above
