@@ -1,6 +1,6 @@
 /* Output the generated parsing program for Bison.
 
-   Copyright (C) 1984, 1986, 1989, 1992, 2000-2006, 2009-2015 Free
+   Copyright (C) 1984, 1986, 1989, 1992, 2000-2006, 2009-2015, 2018 Free
    Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -698,7 +698,7 @@ pack_vector (vector_number vector)
 
       if (ok)
         {
-          int loc;
+          int loc PACIFY_CC (= -1);
           int k;
           for (k = 0; k < t; k++)
             {
